@@ -1,8 +1,16 @@
 {-# LANGUAGE NoImplicitPrelude   #-}
+{-# LANGUAGE LambdaCase          #-}
 {-# LANGUAGE OverloadedRecordDot #-}
 {-# LANGUAGE OverloadedStrings   #-}
 
--- | Generate haddocks
+{-|
+Module      : Stack.Build.Haddock
+Description : Generate Haddock documentation.
+License     : BSD-3-Clause
+
+Generate Haddock documentation.
+-}
+
 module Stack.Build.Haddock
   ( generateDepsHaddockIndex
   , generateLocalHaddockIndex
@@ -20,22 +28,22 @@ import qualified Data.HashSet as HS
 import qualified Data.Map.Strict as Map
 import qualified Data.Set as Set
 import qualified Data.Text as T
-import           Data.Time ( UTCTime )
 import           Distribution.Text ( display )
 import           Path
-                   ( (</>), addExtension, fromAbsDir, fromAbsFile, fromRelDir
-                   , parent, parseRelDir, parseRelFile
+                   ( (</>), addExtension, dirname, fileExtension, filename
+                   , fromAbsDir, fromAbsFile, fromRelDir, parent, parseRelDir
+                   , parseRelFile
                    )
 import           Path.Extra
                    ( parseCollapsedAbsFile, toFilePathNoTrailingSep
                    , tryGetModificationTime
                    )
 import           Path.IO
-                   ( copyDirRecur', doesFileExist, ensureDir, ignoringAbsence
-                   , removeDirRecur
+                   ( copyDirRecur, copyDirRecur', doesDirExist, doesFileExist
+                   , ensureDir, ignoringAbsence, listDir, removeDirRecur
                    )
 import qualified RIO.ByteString.Lazy as BL
-import           RIO.List ( intercalate )
+import           RIO.List ( intercalate, intersperse )
 import           RIO.Process ( HasProcessContext, withWorkingDir )
 import           Stack.Constants
                    ( docDirSuffix, htmlDirSuffix, relDirAll, relFileIndexHtml )
@@ -46,14 +54,16 @@ import           Stack.Types.CompilerPaths
                    ( CompilerPaths (..), HasCompiler (..) )
 import           Stack.Types.ConfigureOpts ( BaseConfigOpts (..) )
 import           Stack.Types.BuildOpts ( BuildOpts (..), HaddockOpts (..) )
-import           Stack.Types.BuildOptsCLI ( BuildOptsCLI (..) )
+import           Stack.Types.BuildOptsCLI ( BuildOptsCLI (..), BuildSubset (BSOnlyDependencies, BSOnlySnapshot) )
 import           Stack.Types.DumpPackage ( DumpPackage (..) )
-import           Stack.Types.EnvConfig ( HasEnvConfig (..) )
+import           Stack.Types.EnvConfig ( EnvConfig (..), HasEnvConfig (..) )
 import           Stack.Types.GhcPkgId ( GhcPkgId )
+import           Stack.Types.InterfaceOpt ( InterfaceOpt (..) )
 import           Stack.Types.Package
                    ( InstallLocation (..), LocalPackage (..), Package (..) )
 import qualified System.FilePath as FP
 import           Web.Browser ( openBrowser )
+import RIO.FilePath (dropTrailingPathSeparator)
 
 openHaddocksInBrowser ::
      HasTerm env
@@ -97,8 +107,7 @@ openHaddocksInBrowser bco pkgLocations buildTargets = do
               getDocIndex
       _ -> getDocIndex
   prettyInfo $ "Opening" <+> pretty docFile <+> "in the browser."
-  _ <- liftIO $ openBrowser (toFilePath docFile)
-  pure ()
+  void $ liftIO $ openBrowser (toFilePath docFile)
 
 -- | Determine whether we should haddock for a package.
 shouldHaddockPackage ::
@@ -117,7 +126,7 @@ shouldHaddockPackage bopts wanted name =
 shouldHaddockDeps :: BuildOpts -> Bool
 shouldHaddockDeps bopts = fromMaybe bopts.buildHaddocks bopts.haddockDeps
 
--- | Generate Haddock index and contents for local packages.
+-- | Generate Haddock index and contents for project packages.
 generateLocalHaddockIndex ::
      (HasCompiler env, HasProcessContext env, HasTerm env)
   => BaseConfigOpts
@@ -136,13 +145,13 @@ generateLocalHaddockIndex bco localDumpPkgs locals = do
           )
           locals
   generateHaddockIndex
-    "local packages"
+    "project packages"
     bco
     dumpPackages
     "."
     (localDocDir bco)
 
--- | Generate Haddock index and contents for local packages and their
+-- | Generate Haddock index and contents for project packages and their
 -- dependencies.
 generateDepsHaddockIndex ::
      (HasCompiler env, HasProcessContext env, HasTerm env)
@@ -161,7 +170,7 @@ generateDepsHaddockIndex bco globalDumpPkgs snapshotDumpPkgs localDumpPkgs local
              ) locals
       depDocDir = localDepsDocDir bco
   generateHaddockIndex
-    "local packages and dependencies"
+    "project packages and dependencies"
     bco
     deps
     ".."
@@ -223,7 +232,9 @@ generateHaddockIndex descr bco dumpPackages docRelFP destDir = do
           case eindexModTime of
             Left _ -> True
             Right indexModTime ->
-              or [mt > indexModTime | (_, mt, _, _) <- interfaceOpts]
+              or [ mt > indexModTime
+                 | mt <- map (.srcInterfaceFileModTime) interfaceOpts
+                 ]
         prettyDescr = style Current (fromString $ T.unpack descr)
     if needUpdate
       then do
@@ -244,7 +255,7 @@ generateHaddockIndex descr bco dumpPackages docRelFP destDir = do
                  [bco.snapDB, bco.localDB]
               ++ bco.buildOpts.haddockOpts.additionalArgs
               ++ ["--gen-contents", "--gen-index"]
-              ++ [x | (xs, _, _, _) <- interfaceOpts, x <- xs]
+              ++ [x | xs <- map (.readInterfaceArgs) interfaceOpts, x <- xs]
           )
       else
         prettyInfo $
@@ -258,61 +269,119 @@ generateHaddockIndex descr bco dumpPackages docRelFP destDir = do
  where
   toInterfaceOpt ::
        DumpPackage
-    -> IO (Maybe ([String], UTCTime, Path Abs File, Path Abs File))
-  toInterfaceOpt DumpPackage {haddockInterfaces, packageIdent, haddockHtml} =
-    case haddockInterfaces of
+    -> IO (Maybe InterfaceOpt)
+  toInterfaceOpt dp =
+    case dp.haddockInterfaces of
       [] -> pure Nothing
       srcInterfaceFP:_ -> do
-        srcInterfaceAbsFile <- parseCollapsedAbsFile srcInterfaceFP
-        let (PackageIdentifier name _) = packageIdent
+        srcInterfaceFile <- parseCollapsedAbsFile srcInterfaceFP
+        let (PackageIdentifier name _) = dp.packageIdent
+            srcInterfaceDir = parent srcInterfaceFile
+        compInterfaceDirsAndFiles <- do
+          -- It is possible that the *.haddock file specified by the
+          -- haddock-interfaces key for an installed package may not exist. For
+          -- example, with GHC 9.6.6 on Windows, there is no
+          --
+          -- ${pkgroot}/../doc/html/libraries/rts-1.0.2\rts.haddock
+          (srcInterfaceSubDirs, _) <- doesDirExist srcInterfaceDir >>= \case
+            True -> listDir srcInterfaceDir
+            False -> pure ([], [])
+          -- This assumes that Cabal (the library) `haddock --executables` for
+          -- component my-component of package my-package puts one *.haddock
+          -- file and associated files in directory:
+          --
+          -- my-package/my-component
+          --
+          -- Not all directories in directory my-package relate to components.
+          -- For example, my-package/src relates to the files for the
+          -- colourised code of the main library of package my-package.
+          let isCompInterfaceDir dir = do
+                (_, files) <- listDir dir
+                pure $ (dir, ) <$> F.find isInterface files
+               where
+                isInterface file = fileExtension file == Just ".haddock"
+          mapMaybeM isCompInterfaceDir srcInterfaceSubDirs
+        -- Lift a copy of the component's Haddock directory up to the same level
+        -- as the main library's Haddock directory. For compontent my-component
+        -- of package my-package we name the directory my-package_my-component.
+        let liftcompInterfaceDir dir file = do
+              let parentDir = parent dir
+                  parentName = dirname parentDir
+                  compName = dirname dir
+              uniqueName <- do
+                let parentName' =
+                      dropTrailingPathSeparator $ toFilePath parentName
+                    compName' =
+                      dropTrailingPathSeparator $ toFilePath compName
+                parseRelDir $ parentName' <> "_" <> compName'
+              let destCompDir = parent parentDir </> uniqueName
+                  destCompFile = destCompDir </> filename file
+              ignoringAbsence (removeDirRecur destCompDir)
+              ensureDir destCompDir
+              onException
+                (copyDirRecur dir destCompDir)
+                (ignoringAbsence (removeDirRecur destCompDir))
+              pure (destCompFile, uniqueName)
             destInterfaceRelFP =
               docRelFP FP.</>
-              packageIdentifierString packageIdent FP.</>
+              packageIdentifierString dp.packageIdent FP.</>
               (packageNameString name FP.<.> "haddock")
             docPathRelFP =
-              fmap ((docRelFP FP.</>) . FP.takeFileName) haddockHtml
-            interfaces = intercalate "," $ mcons docPathRelFP [srcInterfaceFP]
-
-        destInterfaceAbsFile <-
+              fmap ((docRelFP FP.</>) . FP.takeFileName) dp.haddockHtml
+            mkInterface :: Maybe FilePath -> FilePath -> String
+            mkInterface mDocPath file =
+              intercalate "," $ mcons mDocPath [file]
+            compInterface :: (Path Abs Dir, Path Abs File) -> IO String
+            compInterface (dir, file) = do
+              (file', uniqueName) <- liftcompInterfaceDir dir file
+              let compDir = dropTrailingPathSeparator $ toFilePath uniqueName
+                  docDir = docRelFP FP.</> compDir
+              pure $ mkInterface (Just docDir) (toFilePath file')
+            interfaces = mkInterface docPathRelFP srcInterfaceFP
+        compInterfaces <- forM compInterfaceDirsAndFiles compInterface
+        let readInterfaceArgs =
+              "-i" : intersperse "-i" (interfaces : compInterfaces)
+        destInterfaceFile <-
           parseCollapsedAbsFile (toFilePath destDir FP.</> destInterfaceRelFP)
-        esrcInterfaceModTime <- tryGetModificationTime srcInterfaceAbsFile
+        eSrcInterfaceFileModTime <- tryGetModificationTime srcInterfaceFile
         pure $
-          case esrcInterfaceModTime of
+          case eSrcInterfaceFileModTime of
             Left _ -> Nothing
-            Right srcInterfaceModTime ->
-              Just
-                ( [ "-i", interfaces ]
-                , srcInterfaceModTime
-                , srcInterfaceAbsFile
-                , destInterfaceAbsFile
-                )
-  copyPkgDocs :: (a, UTCTime, Path Abs File, Path Abs File) -> IO ()
-  copyPkgDocs (_, srcInterfaceModTime, srcInterfaceAbsFile, destInterfaceAbsFile) = do
+            Right srcInterfaceFileModTime ->
+              Just InterfaceOpt
+                { readInterfaceArgs
+                , srcInterfaceFileModTime
+                , srcInterfaceFile
+                , destInterfaceFile
+                }
+  copyPkgDocs :: InterfaceOpt -> IO ()
+  copyPkgDocs opts = do
   -- Copy dependencies' haddocks to documentation directory.  This way,
   -- relative @../$pkg-$ver@ links work and it's easy to upload docs to a web
   -- server or otherwise view them in a non-local-filesystem context. We copy
   -- instead of symlink for two reasons: (1) symlinks aren't reliably supported
   -- on Windows, and (2) the filesystem containing dependencies' docs may not be
   -- available where viewing the docs (e.g. if building in a Docker container).
-    edestInterfaceModTime <- tryGetModificationTime destInterfaceAbsFile
+    edestInterfaceModTime <- tryGetModificationTime opts.destInterfaceFile
     case edestInterfaceModTime of
       Left _ -> doCopy
       Right destInterfaceModTime
-        | destInterfaceModTime < srcInterfaceModTime -> doCopy
+        | destInterfaceModTime < opts.srcInterfaceFileModTime -> doCopy
         | otherwise -> pure ()
    where
     doCopy = do
       ignoringAbsence (removeDirRecur destHtmlAbsDir)
       ensureDir destHtmlAbsDir
       onException
-        (copyDirRecur' (parent srcInterfaceAbsFile) destHtmlAbsDir)
+        (copyDirRecur' (parent opts.srcInterfaceFile) destHtmlAbsDir)
         (ignoringAbsence (removeDirRecur destHtmlAbsDir))
-    destHtmlAbsDir = parent destInterfaceAbsFile
+    destHtmlAbsDir = parent opts.destInterfaceFile
 
 -- | Find first DumpPackage matching the GhcPkgId
-lookupDumpPackage :: GhcPkgId
-                  -> [Map GhcPkgId DumpPackage]
-                  -> Maybe DumpPackage
+lookupDumpPackage ::
+     GhcPkgId
+  -> [Map GhcPkgId DumpPackage]
+  -> Maybe DumpPackage
 lookupDumpPackage ghcPkgId dumpPkgs =
   listToMaybe $ mapMaybe (Map.lookup ghcPkgId) dumpPkgs
 
@@ -320,11 +389,11 @@ lookupDumpPackage ghcPkgId dumpPkgs =
 haddockIndexFile :: Path Abs Dir -> Path Abs File
 haddockIndexFile destDir = destDir </> relFileIndexHtml
 
--- | Path of local packages documentation directory.
+-- | Path of project packages documentation directory.
 localDocDir :: BaseConfigOpts -> Path Abs Dir
 localDocDir bco = bco.localInstallRoot </> docDirSuffix
 
--- | Path of documentation directory for the dependencies of local packages
+-- | Path of documentation directory for the dependencies of project packages
 localDepsDocDir :: BaseConfigOpts -> Path Abs Dir
 localDepsDocDir bco = localDocDir bco </> relDirAll
 
@@ -336,14 +405,17 @@ generateLocalHaddockForHackageArchives ::
      (HasEnvConfig env, HasTerm env)
   => [LocalPackage]
   -> RIO env ()
-generateLocalHaddockForHackageArchives =
-  mapM_
-    ( \lp ->
-        let pkg = lp.package
-            pkgId = PackageIdentifier pkg.name pkg.version
-            pkgDir = parent lp.cabalFP
-        in generateLocalHaddockForHackageArchive pkgDir pkgId
-    )
+generateLocalHaddockForHackageArchives lps = do
+  buildSubset <- view $ envConfigL . to (.buildOptsCLI.buildSubset)
+  let localsExcluded =
+        buildSubset == BSOnlyDependencies || buildSubset == BSOnlySnapshot
+  unless localsExcluded $
+    forM_ lps $ \lp ->
+      let pkg = lp.package
+          pkgId = PackageIdentifier pkg.name pkg.version
+          pkgDir = parent lp.cabalFP
+      in  when lp.wanted $
+            generateLocalHaddockForHackageArchive pkgDir pkgId
 
 -- | Generate an archive file containing local Haddock documentation for
 -- Hackage, in a form accepted by Hackage.
@@ -367,15 +439,22 @@ generateLocalHaddockForHackageArchive pkgDir pkgId = do
         )
       tarGzFile = distDir </> tarGzFileName
       docDir = distDir </> docDirSuffix </> htmlDirSuffix
-  createTarGzFile tarGzFile docDir nameRelDir
-  prettyInfo $
-       fillSep
-         [ flow "Archive of Haddock documentation for Hackage for"
-         , style Current (fromString pkgIdName)
-         , flow "created at:"
-         ]
-    <> line
-    <> pretty tarGzFile
+  tarGzFileCreated <- createTarGzFile tarGzFile docDir nameRelDir
+  if tarGzFileCreated
+    then
+      prettyInfo $
+           fillSep
+             [ flow "Archive of Haddock documentation for Hackage for"
+             , style Current (fromString pkgIdName)
+             , flow "created at:"
+             ]
+        <> line
+        <> pretty tarGzFile
+    else
+      prettyWarnL
+        [ flow "No Haddock documentation for Hackage available for"
+        , style Error (fromString pkgIdName) <> "."
+        ]
 
 createTarGzFile ::
      Path Abs File
@@ -384,10 +463,19 @@ createTarGzFile ::
      -- ^ Base directory
   -> Path Rel Dir
      -- ^ Directory to archive, relative to base directory
-  -> RIO env ()
+  -> RIO env Bool
 createTarGzFile tar base dir = do
-   entries <- liftIO $ Tar.pack base' [dir']
-   BL.writeFile tar' $ GZip.compress $ Tar.write entries
+  dirExists <- doesDirExist $ base </> dir
+  if dirExists
+    then do
+      entries <- liftIO $ Tar.pack base' [dir']
+      if null entries
+        then pure False
+        else do
+          ensureDir $ parent tar
+          BL.writeFile tar' $ GZip.compress $ Tar.write entries
+          pure True
+    else pure False
  where
   base' = fromAbsDir base
   dir' = fromRelDir dir
