@@ -1,5 +1,13 @@
 {-# LANGUAGE NoImplicitPrelude #-}
 
+{-|
+Module      : Stack.Constants.StackProgName
+License     : BSD-3-Clause
+
+'stackProgName' is defined in this module rather than in "Stack.Constants", due
+to GHC stage restrictions and the use of Template Haskell.
+-}
+
 module Stack.Constants.StackProgName
   ( stackProgName
   ) where
@@ -7,8 +15,5 @@ module Stack.Constants.StackProgName
 import           Stack.Prelude ( String )
 
 -- | Name of the Stack program.
-
--- NOTE: Defined in this module rather than in "Stack.Constants", due to
--- GHC stage restrictions and the use of Template Haskell.
 stackProgName :: String
 stackProgName = "stack"
