@@ -3,7 +3,14 @@
 {-# LANGUAGE NoFieldSelectors      #-}
 {-# LANGUAGE OverloadedRecordDot   #-}
 
--- | Types and functions related to Stack's @eval@ command.
+{-|
+Module      : Stack.Eval
+Description : Types and functions related to Stack's @eval@ command.
+License     : BSD-3-Clause
+
+Types and functions related to Stack's @eval@ command.
+-}
+
 module Stack.Eval
   ( EvalOpts (..)
   , evalCmd
@@ -16,7 +23,7 @@ import           Stack.Exec
 import           Stack.Prelude
 import           Stack.Types.Runner ( Runner )
 
--- Type representing command line options for the @stack eval@ command.
+-- | Type representing command line options for the @stack eval@ command.
 data EvalOpts = EvalOpts
   { arg :: !String
   , extra :: !ExecOptsExtra
