@@ -1,5 +1,11 @@
-{-# LANGUAGE NoImplicitPrelude #-}
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE NoImplicitPrelude   #-}
+{-# LANGUAGE OverloadedRecordDot #-}
+{-# LANGUAGE OverloadedStrings   #-}
+
+{-|
+Module      : Stack.FileWatch
+License     : BSD-3-Clause
+-}
 
 module Stack.FileWatch
   ( WatchMode (WatchModePoll)
@@ -11,10 +17,19 @@ import           Control.Concurrent.STM ( check )
 import qualified Data.Map.Merge.Strict as Map
 import qualified Data.Map.Strict as Map
 import qualified Data.Set as Set
+import qualified Data.Text as T
 import           GHC.IO.Exception
                    ( IOErrorType (InvalidArgument), IOException (..) )
-import           Path ( parent )
+import           Path ( fileExtension, parent )
+import           Path.IO ( doesFileExist, executable, getPermissions )
+import           RIO.Process
+                   ( EnvVars, HasProcessContext (..), proc, runProcess
+                   , withModifyEnvVars
+                   )
+import           System.Permissions ( osIsWindows )
 import           Stack.Prelude
+import           Stack.Types.Config ( Config (..), HasConfig (..) )
+import           Stack.Types.Runner ( HasRunner (..), Runner (..) )
 import           System.FSNotify
                    ( WatchConfig, WatchMode (..), confWatchMode, defaultConfig
                    , eventPath, watchDir, withManagerConf
@@ -22,14 +37,14 @@ import           System.FSNotify
 import           System.IO ( getLine )
 
 fileWatch ::
-     HasTerm env
-  => ((Set (Path Abs File) -> IO ()) -> RIO env ())
+     (HasConfig env, HasTerm env)
+  => ((Set (Path Abs File) -> IO ()) -> RIO Runner ())
   -> RIO env ()
 fileWatch = fileWatchConf defaultConfig
 
 fileWatchPoll ::
-     HasTerm env
-  => ((Set (Path Abs File) -> IO ()) -> RIO env ())
+     (HasConfig env, HasTerm env)
+  => ((Set (Path Abs File) -> IO ()) -> RIO Runner ())
   -> RIO env ()
 fileWatchPoll =
   fileWatchConf $ defaultConfig { confWatchMode = WatchModePoll 1000000 }
@@ -39,11 +54,13 @@ fileWatchPoll =
 -- The action provided takes a callback that is used to set the files to be
 -- watched. When any of those files are changed, we rerun the action again.
 fileWatchConf ::
-     HasTerm env
+     (HasConfig env, HasTerm env)
   => WatchConfig
-  -> ((Set (Path Abs File) -> IO ()) -> RIO env ())
+  -> ((Set (Path Abs File) -> IO ()) -> RIO Runner ())
   -> RIO env ()
-fileWatchConf cfg inner =
+fileWatchConf cfg inner = do
+  runner <- view runnerL
+  mHook <- view $ configL . to (.fileWatchHook)
   withRunInIO $ \run -> withManagerConf cfg $ \manager -> do
     allFiles <- newTVarIO Set.empty
     dirtyVar <- newTVarIO True
@@ -134,7 +151,7 @@ fileWatchConf cfg inner =
         dirty <- readTVar dirtyVar
         check dirty
 
-      eres <- tryAny $ inner setWatched
+      eres <- tryAny $ runRIO runner (inner setWatched)
 
       -- Clear dirtiness flag after the build to avoid an infinite loop caused
       -- by the build itself triggering dirtiness. This could be viewed as a
@@ -143,19 +160,63 @@ fileWatchConf cfg inner =
       -- https://github.com/commercialhaskell/stack/issues/822
       atomically $ writeTVar dirtyVar False
 
-      case eres of
-        Left e ->
-          case fromException e of
-            Just ExitSuccess ->
-              prettyInfo $ style Good $ fromString $ displayException e
-            _ -> case fromException e :: Maybe PrettyException of
-              Just pe -> prettyError $ pretty pe
-              _ -> prettyInfo $ style Error $ fromString $ displayException e
-        _ -> prettyInfo $
-               style Good (flow "Success! Waiting for next file change.")
+      let defaultAction = case eres of
+            Left e ->
+              case fromException e of
+                Just ExitSuccess ->
+                  prettyInfo $ style Good $ fromString $ displayException e
+                _ -> case fromException e :: Maybe PrettyException of
+                  Just pe -> prettyError $ pretty pe
+                  _ -> prettyInfo $ style Error $ fromString $ displayException e
+            _ -> prettyInfo $
+                   style Good (flow "Success! Waiting for next file change.")
+
+      case mHook of
+        Nothing -> defaultAction
+        Just hook -> do
+          hookIsExecutable <- handleIO (\_ -> pure False) $ if osIsWindows
+            then
+              -- can't really detect executable on windows, only file extension
+              doesFileExist hook
+            else executable <$> getPermissions hook
+          if hookIsExecutable
+            then runFileWatchHook eres hook
+            else do
+              prettyWarn $
+                flow "File watch hook not executable. Falling back on default."
+              defaultAction
 
       prettyInfoL
         [ "Type"
         , style Shell "help"
         , flow "for the available commands. Press enter to force a rebuild."
         ]
+
+runFileWatchHook ::
+     (HasProcessContext env, HasTerm env)
+  => Either SomeException ()
+  -> Path Abs File
+  -> RIO env ()
+runFileWatchHook buildResult hook =
+  withModifyEnvVars insertBuildResultInEnv $ do
+    let (cmd, args) = if osIsWindows && isShFile
+          then ("sh", [toFilePath hook])
+          else (toFilePath hook, [])
+    menv <- view processContextL
+    exit <- withProcessContext menv $ proc cmd args runProcess
+    case exit of
+      ExitSuccess -> pure ()
+      ExitFailure i -> do
+        prettyWarnL
+          [ flow "File watch hook exited with code:"
+          , style Error (fromString $ show i) <> "."
+          ]
+        pure ()
+ where
+  insertBuildResultInEnv :: EnvVars -> EnvVars
+  insertBuildResultInEnv = Map.insert "HOOK_FW_RESULT" $ case buildResult of
+    Left e -> T.pack $ displayException e
+    Right _ -> ""
+  isShFile = case fileExtension hook of
+    Just ".sh" -> True
+    _ -> False
