@@ -3,6 +3,11 @@
 {-# LANGUAGE OverloadedRecordDot #-}
 {-# LANGUAGE OverloadedStrings   #-}
 
+{-|
+Module      : Stack.Lock
+License     : BSD-3-Clause
+-}
+
 module Stack.Lock
   ( lockCachedWanted
   , LockedLocation (..)
@@ -19,8 +24,7 @@ import qualified Data.Map as Map
 import qualified Data.Text as T
 import qualified Data.Yaml as Yaml
 import qualified RIO.NonEmpty as NE
-import           Path ( parent )
-import           Path.Extended ( addExtension )
+import           Path ( addExtension, parent )
 import           Path.IO ( doesFileExist )
 import           Stack.Prelude
 import           Stack.SourceMap ( snapToDepPackage )
@@ -52,6 +56,7 @@ instance Pretty LockPrettyException where
 
 instance Exception LockPrettyException
 
+-- | Type representing locked locations.
 data LockedLocation a b = LockedLocation
   { original :: a
   , completed :: b
@@ -85,6 +90,7 @@ instance FromJSON (WithJSONWarnings (Unresolved SingleRPLI)) where
       let withWarnings x = WithJSONWarnings x ws
       pure $ withWarnings $ SingleRPLI . NE.head <$> unresolvedRPLIs
 
+-- | Type representing the contents of lock files.
 data Locked = Locked
   { snapshotLocations :: [LockedLocation RawSnapshotLocation SnapshotLocation]
   , pkgImmutableLocations :: [LockedLocation RawPackageLocationImmutable PackageLocationImmutable]
@@ -122,17 +128,21 @@ loadYamlThrow parser path = do
         logJSONWarnings (toFilePath path) warnings
         pure res
 
+-- | Yields a t'Stack.Types.SourceMap.SMWanted' taking into account the relevant
+-- lock file, if one is applicable and it exists.
 lockCachedWanted ::
      (HasPantryConfig env, HasRunner env)
   => Path Abs File
+     -- ^ The relevant Stack project-level configuration file.
   -> RawSnapshotLocation
+     -- ^ The relevant snapshot.
   -> (  Map RawPackageLocationImmutable PackageLocationImmutable
      -> WantedCompiler
      -> Map PackageName (Bool -> RIO env DepPackage)
      -> RIO env ( SMWanted, [CompletedPLI])
      )
   -> RIO env SMWanted
-lockCachedWanted stackFile resolver fillWanted = do
+lockCachedWanted stackFile snapshot fillWanted = do
   lockFile <- liftIO $ addExtension ".lock" stackFile
   let getLockExists = doesFileExist lockFile
   lfb <- view lockFileBehaviorL
@@ -157,7 +167,7 @@ lockCachedWanted stackFile resolver fillWanted = do
       pkgLocCache = toMap locked.pkgImmutableLocations
   debugRSL <- view rslInLogL
   (snap, slocCompleted, pliCompleted) <-
-    loadAndCompleteSnapshotRaw' debugRSL resolver slocCache pkgLocCache
+    loadAndCompleteSnapshotRaw' debugRSL snapshot slocCache pkgLocCache
   let compiler = snapshotCompiler snap
       snPkgs = Map.mapWithKey
                  (\n p h -> snapToDepPackage h n p)
@@ -188,4 +198,4 @@ lockCachedWanted stackFile resolver fillWanted = do
     "# This file was autogenerated by Stack.\n\
     \# You should not edit this file by hand.\n\
     \# For more information, please see the documentation at:\n\
-    \#   https://docs.haskellstack.org/en/stable/lock_files\n\n"
+    \#   https://docs.haskellstack.org/en/stable/topics/lock_files\n\n"
