@@ -2,6 +2,14 @@
 {-# LANGUAGE OverloadedRecordDot #-}
 {-# LANGUAGE OverloadedStrings   #-}
 
+{-|
+Module      : Stack.Options.Completion
+Description : Completers for command line arguments.
+License     : BSD-3-Clause
+
+Completers for command line arguments or arguments of command line options.
+-}
+
 module Stack.Options.Completion
   ( ghcOptsCompleter
   , targetCompleter
@@ -33,6 +41,7 @@ import           Stack.Types.ProjectConfig ( ProjectConfig (..) )
 import           Stack.Types.NamedComponent ( renderPkgComponent )
 import           Stack.Types.SourceMap ( SMWanted (..), ppComponents, ppGPD )
 
+-- | A completer for @--ghc-options@ or @--ghci-options@.
 ghcOptsCompleter :: Completer
 ghcOptsCompleter = mkCompleter $ \inputRaw -> pure $
   let input = unescapeBashArg inputRaw
@@ -48,19 +57,21 @@ ghcOptsCompleter = mkCompleter $ \inputRaw -> pure $
 -- TODO: Ideally this would pay attention to --stack-yaml, may require
 -- changes to optparse-applicative.
 
-buildConfigCompleter ::
-     (String -> RIO EnvConfig [String])
-  -> Completer
+buildConfigCompleter :: (String -> RIO EnvConfig [String]) -> Completer
 buildConfigCompleter inner = mkCompleter $ \inputRaw -> do
   let input = unescapeBashArg inputRaw
   case input of
     -- If it looks like a flag, skip this more costly completion.
     ('-': _) -> pure []
     _ -> do
-      go' <- globalOptsFromMonoid False mempty
+      -- We do not need to specify the name of the current Stack executable, as
+      -- it was invoked, or the path to the current Stack executable, as
+      -- withDefaultEnvConfig does not need either.
+      go' <- globalOptsFromMonoid "" Nothing False mempty
       let go = go' { logLevel = LevelOther "silent" }
       withRunnerGlobal go $ withConfig NoReexec $ withDefaultEnvConfig $ inner input
 
+-- | A completer for components of project packages.
 targetCompleter :: Completer
 targetCompleter = buildConfigCompleter $ \input -> do
   packages <- view $ buildConfigL . to (.smWanted.project)
@@ -73,6 +84,7 @@ targetCompleter = buildConfigCompleter $ \input -> do
   allComponentNames (name, comps) =
     map (T.unpack . renderPkgComponent . (name,)) (Set.toList comps)
 
+-- | A completer for Cabal flags of project packages.
 flagCompleter :: Completer
 flagCompleter = buildConfigCompleter $ \input -> do
   bconfig <- view buildConfigL
@@ -105,6 +117,7 @@ flagCompleter = buildConfigCompleter $ \input -> do
       ('*' : _) -> wildcardFlags
       _ -> normalFlags
 
+-- | A completer for executable components of project packages.
 projectExeCompleter :: Completer
 projectExeCompleter = buildConfigCompleter $ \input -> do
   packages <- view $ buildConfigL . to (.smWanted.project)
