@@ -1,22 +1,24 @@
 {-# LANGUAGE NoImplicitPrelude #-}
 
+{-|
+Module      : Stack.Options.HpcReportParser
+Description : Parser for @stack hpc report@.
+License     : BSD-3-Clause
+
+Parser for @stack hpc report@.
+-}
+
 module Stack.Options.HpcReportParser
   ( hpcReportOptsParser
-  , pvpBoundsOption
   ) where
 
-import qualified Data.Text as T
 import           Options.Applicative
-                   ( Parser, completer, completeWith, help, long, metavar
-                   , option, readerError, strOption, switch
-                   )
+                   ( Parser, completer, help, long, metavar, strOption, switch )
 import           Options.Applicative.Builder.Extra
                    ( dirCompleter, fileExtCompleter, textArgument )
-import           Options.Applicative.Types ( readerAsk )
-import           Stack.Coverage ( HpcReportOpts (..) )
 import           Stack.Options.Completion ( targetCompleter )
 import           Stack.Prelude
-import           Stack.Types.PvpBounds ( PvpBounds, parsePvpBounds )
+import           Stack.Types.HpcReportOpts ( HpcReportOpts (..) )
 
 -- | Parser for @stack hpc report@.
 hpcReportOptsParser :: Parser HpcReportOpts
@@ -40,18 +42,3 @@ hpcReportOptsParser = HpcReportOpts
         (  long "open"
         <> help "Open the report in the browser."
         )
-
-pvpBoundsOption :: Parser PvpBounds
-pvpBoundsOption = option readPvpBounds
-  (  long "pvp-bounds"
-  <> metavar "PVP-BOUNDS"
-  <> completeWith ["none", "lower", "upper", "both"]
-  <> help "How PVP version bounds should be added to Cabal file: none, lower, \
-          \upper, both."
-  )
- where
-  readPvpBounds = do
-    s <- readerAsk
-    case parsePvpBounds $ T.pack s of
-      Left e -> readerError e
-      Right v -> pure v
