@@ -1,6 +1,11 @@
 {-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE NoFieldSelectors  #-}
 
+{-|
+Module      : Stack.Types.GhcOptions
+License     : BSD-3-Clause
+-}
+
 module Stack.Types.GhcOptions
   ( GhcOptions (..)
   ) where
@@ -10,6 +15,7 @@ import           Data.Attoparsec.Args ( EscapingMode (Escaping), parseArgs )
 import qualified Data.Text as T
 import           Stack.Prelude
 
+-- | Type representing lists of GHC options.
 newtype GhcOptions = GhcOptions { ghcOptions :: [Text] }
 
 instance FromJSON GhcOptions where
