@@ -3,11 +3,17 @@
 {-# LANGUAGE NoFieldSelectors      #-}
 {-# LANGUAGE OverloadedRecordDot   #-}
 
--- | A sourcemap maps a package name to how it should be built, including source
--- code, flags, options, etc. This module contains various stages of source map
--- construction. See the @build_overview.md@ doc for details on these stages.
+{-|
+Module      : Stack.Types.SourceMap
+License     : BSD-3-Clause
+
+A source map maps a package name to how it should be built, including source
+code, flags and options. This module exports types used in various stages of
+source map construction. See @build_overview.md@ for details on these stages.
+-}
+
 module Stack.Types.SourceMap
-  ( -- * Different source map types
+  ( -- * Source map types
     SMWanted (..)
   , SMActual (..)
   , Target (..)
@@ -39,6 +45,7 @@ import qualified Pantry.SHA256 as SHA256
 import           Path ( parent, parseRelDir )
 import           Stack.Prelude
 import           Stack.Types.Compiler ( ActualCompiler )
+import           Stack.Types.ComponentUtils ( fromCabalName )
 import           Stack.Types.NamedComponent ( NamedComponent (..) )
 
 -- | Settings common to dependency packages ('Stack.Types.SourceMap.DepPackage')
@@ -55,8 +62,8 @@ data CommonPackage = CommonPackage
     -- ^ Should Haddock documentation be built for this package?
   }
 
--- | Flag showing if package comes from a snapshot needed to ignore dependency
--- bounds between such packages
+-- | Flag showing if package comes from a snapshot. Used to ignore dependency
+-- bounds between such packages.
 data FromSnapshot
   = FromSnapshot
   | NotFromSnapshot
@@ -74,16 +81,20 @@ data DepPackage = DepPackage
     -- See https://github.com/commercialhaskell/stackage/issues/3185
   }
 
--- | A view of a project package needed for resolving components
+-- | A view of a project package. Used to resolve components.
 data ProjectPackage = ProjectPackage
   { projectCommon :: !CommonPackage
   , cabalFP :: !(Path Abs File)
   , resolvedDir :: !(ResolvedPath Dir)
   }
 
--- | A view of a package installed in the global package database also could
--- include marker for a replaced global package (could be replaced because of a
--- replaced dependency)
+-- | A type representing versions of packages in the global package database.
+newtype GlobalPackageVersion
+  = GlobalPackageVersion Version
+
+-- | A view of a package installed in the global package database or a marker
+-- for a replaced global package. A global package could be replaced because of
+-- a replaced dependency.
 data GlobalPackage
   = GlobalPackage !Version
   | ReplacedGlobalPackage ![PackageName]
@@ -93,14 +104,29 @@ isReplacedGlobal :: GlobalPackage -> Bool
 isReplacedGlobal (ReplacedGlobalPackage _) = True
 isReplacedGlobal (GlobalPackage _) = False
 
+-- | A type representing how a package is intended to be built.
+data Target
+  = TargetAll !PackageType
+    -- ^ Build all of the default components.
+  | TargetComps !(Set NamedComponent)
+    -- ^ Only build specific components
+
+-- | A type representing types of packages.
+data PackageType
+  = PTProject
+    -- ^ The package is a project package.
+  | PTDependency
+    -- ^ The package is other than a project package and a dependency.
+  deriving (Eq, Show)
+
 -- | A source map with information on the wanted (but not actual) compiler. This
 -- is derived by parsing the @stack.yaml@ file for @packages@, @extra-deps@,
 -- their configuration (e.g., flags and options), and parsing the snapshot it
 -- refers to. It does not include global packages or any information from the
 -- command line.
 --
--- Invariant: a @PackageName@ appears in either 'smwProject' or 'smwDeps', but
--- not both.
+-- Invariant: a @PackageName@ appears in either 'SMWanted.project' or
+-- 'SMWanted.deps', but not both.
 data SMWanted = SMWanted
   { compiler :: !WantedCompiler
   , project :: !(Map PackageName ProjectPackage)
@@ -109,8 +135,9 @@ data SMWanted = SMWanted
     -- ^ Where this snapshot is loaded from.
   }
 
--- | Adds in actual compiler information to 'SMWanted', in particular the
--- contents of the global package database.
+-- | A source map with information on the actual compiler, including the
+-- contents of its global package database. It does not include any information
+-- from the command line.
 --
 -- Invariant: a @PackageName@ appears in only one of the @Map@s.
 data SMActual global = SMActual
@@ -120,28 +147,18 @@ data SMActual global = SMActual
   , globals :: !(Map PackageName global)
   }
 
-newtype GlobalPackageVersion
-  = GlobalPackageVersion Version
-
--- | How a package is intended to be built
-data Target
-  = TargetAll !PackageType
-  -- ^ Build all of the default components.
-  | TargetComps !(Set NamedComponent)
-  -- ^ Only build specific components
-
-data PackageType = PTProject | PTDependency
-  deriving (Eq, Show)
-
--- | Builds on an 'SMActual' by resolving the targets specified on the command
+-- | Builds on an t'SMActual' by resolving the targets specified on the command
 -- line, potentially adding in new dependency packages in the process.
 data SMTargets = SMTargets
   { targets :: !(Map PackageName Target)
   , deps :: !(Map PackageName DepPackage)
   }
 
--- | The final source map, taking an 'SMTargets' and applying all command line
+-- | The final source map, taking an t'SMTargets' and applying all command line
 -- flags and GHC options.
+--
+-- One source map value is distinguished from another by a hash of the parts of
+-- the value that are immutable.
 data SourceMap = SourceMap
   { targets :: !SMTargets
     -- ^ Doesn't need to be included in the hash, does not affect the source
@@ -163,7 +180,7 @@ data SourceMap = SourceMap
     -- hash and make Stack more resilient.
   }
 
--- | A unique hash for the immutable portions of a 'SourceMap'.
+-- | A unique hash for the immutable portions of a t'SourceMap'.
 newtype SourceMapHash
   = SourceMapHash SHA256
 
@@ -174,11 +191,11 @@ smRelDir (SourceMapHash smh) = parseRelDir $ T.unpack $ SHA256.toHexText smh
 ppGPD :: MonadIO m => ProjectPackage -> m GenericPackageDescription
 ppGPD = liftIO . (.projectCommon.gpd)
 
--- | Root directory for the given 'ProjectPackage'
+-- | Root directory for the given t'ProjectPackage'
 ppRoot :: ProjectPackage -> Path Abs Dir
 ppRoot = parent . (.cabalFP)
 
--- | All components available in the given 'ProjectPackage'
+-- | All components available in the given t'ProjectPackage'
 ppComponents :: MonadIO m => ProjectPackage -> m (Set NamedComponent)
 ppComponents = ppComponentsMaybe Just
 
@@ -191,17 +208,13 @@ ppComponentsMaybe compType pp = do
   gpd <- ppGPD pp
   pure $ Set.fromList $ concat
     [ maybe [] (const $ catMaybes [compType CLib]) (C.condLibrary gpd)
-    , go (compType . CExe) (fst <$> C.condExecutables gpd)
-    , go (compType . CTest) (fst <$> C.condTestSuites gpd)
-    , go (compType . CBench) (fst <$> C.condBenchmarks gpd)
+    , mapMaybe ((compType . CExe . fromCabalName) . fst) (C.condExecutables gpd)
+    , mapMaybe ((compType . CTest . fromCabalName) . fst) (C.condTestSuites gpd)
+    , mapMaybe
+        ((compType . CBench . fromCabalName) . fst)
+        (C.condBenchmarks gpd)
     ]
- where
-  go ::
-       (T.Text -> Maybe NamedComponent)
-    -> [C.UnqualComponentName]
-    -> [NamedComponent]
-  go wrapper = mapMaybe (wrapper . T.pack . C.unUnqualComponentName)
 
--- | Version for the given 'ProjectPackage
+-- | Version for the given t'ProjectPackage
 ppVersion :: MonadIO m => ProjectPackage -> m Version
 ppVersion = fmap gpdVersion . ppGPD
