@@ -1,7 +1,13 @@
 {-# LANGUAGE NoImplicitPrelude #-}
 
--- | The module of this name differs as between Windows and non-Windows builds.
--- This is the non-Windows version.
+{-|
+Module      : System.Info.ShortPathName
+License     : BSD-3-Clause
+
+The module of this name differs as between Windows and non-Windows builds. This
+is the non-Windows version.
+-}
+
 module System.Info.ShortPathName
   ( getShortPathName
   ) where
